// Define a name for the test module.
#define BOOST_TEST_MODULE testReadPriodic
// Only after defining the name include the unit test header.
#include <boost/test/included/unit_test.hpp>

#include <ChimeraTK/ApplicationCore/TestFacility.h>
#include "Server.h"
#include "TecDummy.h"


using namespace boost::unit_test_framework;
using namespace ChimeraTK;

BOOST_AUTO_TEST_SUITE(tecTestSuite)

constexpr char TECDummyCDD[] = "(TecDummy:?map=dummydevice.map)";

BOOST_AUTO_TEST_CASE(testReadOnlyInteger){
  std::cout << "testReadOnlyInteger" << std::endl;
  
  Server tec_app{"tec_dummy"};
  ChimeraTK::TestFacility testFacility;
  
  testFacility.runApplication();

  boost::shared_ptr<TecDummy> TECDummy = boost::dynamic_pointer_cast<TecDummy>(
     ChimeraTK::BackendFactory::getInstance().createBackend(TECDummyCDD));

  testFacility.writeScalar<int>("TecModule/mon_serialNumber_enaPeriodicRead", 1);
  tec_app.periodicRead.sendTrigger();
  testFacility.stepApplication();
  auto serialNumberRead = testFacility.getScalar<int>("TecModule/mon_serialNumber_rd");
  serialNumberRead.read();
  //std::cout<<"serialNumber:"<<serialNumber<<std::endl;
  BOOST_CHECK_EQUAL(serialNumberRead,0);

  int serialNumberWritten = 17;
  for(auto i=0;i<10;i++){
    serialNumberWritten = serialNumberWritten* 13;
    TECDummy->set( TecFrame::TecParameter::MonSerialNumber, serialNumberWritten );
    tec_app.periodicRead.sendTrigger();
    testFacility.stepApplication();
    serialNumberRead.read();
    std::cout<<"serialNumberWritten:"<<serialNumberWritten<<"::serialNumberRead:"<<serialNumberRead<<std::endl;
    BOOST_CHECK_EQUAL(static_cast<int>(serialNumberRead),serialNumberWritten);
  }

}

BOOST_AUTO_TEST_SUITE_END()
