// Define a name for the test module.
#define BOOST_TEST_MODULE testWrite
// Only after defining the name include the unit test header.
#include <boost/test/included/unit_test.hpp>
#include <ChimeraTK/ApplicationCore/TestFacility.h>
#include <ChimeraTK/BackendFactory.h>
#include "Server.h"
#include "TecDummy.h"

using namespace boost::unit_test_framework;

/// TestSuite for the server, adapt name
BOOST_AUTO_TEST_SUITE(TecServerTestSuite)

/**********************************************************************************************************************/

/// Test write of an integer
BOOST_AUTO_TEST_CASE(testInteger) {
  std::cout << "testInteger" << std::endl;

  Server theServer{"tec_dummy"};
  ChimeraTK::TestFacility testFacility;

  auto backend = ChimeraTK::BackendFactory::getInstance().createBackend("ActualDevice");
  auto dummy = boost::dynamic_pointer_cast<TecDummy>(backend);

  auto fanControlEnable = testFacility.getScalar<int>("TecModule/ctrl_ch1_fanControlEnable_wr");
  auto writeCh1Group7 = testFacility.getScalar<int>("TecModule/writeCh1Group7_wr");

  testFacility.runApplication();

  fanControlEnable = 1;
  fanControlEnable.write();
  writeCh1Group7.write();
  BOOST_CHECK_EQUAL(dummy->getInt(TecFrame::TecParameter::ExpFanControlEnable), 0);
  testFacility.stepApplication();
  BOOST_CHECK_EQUAL(dummy->getInt(TecFrame::TecParameter::ExpFanControlEnable), 1);

  fanControlEnable = 42;
  fanControlEnable.write();
  writeCh1Group7.write();
  BOOST_CHECK_EQUAL(dummy->getInt(TecFrame::TecParameter::ExpFanControlEnable), 1);
  testFacility.stepApplication();
  BOOST_CHECK_EQUAL(dummy->getInt(TecFrame::TecParameter::ExpFanControlEnable), 42);

  fanControlEnable = -120;
  fanControlEnable.write();
  writeCh1Group7.write();
  BOOST_CHECK_EQUAL(dummy->getInt(TecFrame::TecParameter::ExpFanControlEnable), 42);
  testFacility.stepApplication();
  BOOST_CHECK_EQUAL(dummy->getInt(TecFrame::TecParameter::ExpFanControlEnable), -120);
}

/**********************************************************************************************************************/

/// Test write of a float
BOOST_AUTO_TEST_CASE(testFloat) {
  std::cout << "testFloat" << std::endl;

  Server theServer{"tec_dummy"};
  ChimeraTK::TestFacility testFacility;

  auto backend = ChimeraTK::BackendFactory::getInstance().createBackend("ActualDevice");
  auto dummy = boost::dynamic_pointer_cast<TecDummy>(backend);

  auto targetObjectTemp = testFacility.getScalar<float>("TecModule/ctrl_ch1_targetObjectTemp_wr");
  auto writeCh1Group1 = testFacility.getScalar<int>("TecModule/writeCh1Group1_wr");

  testFacility.runApplication();

  targetObjectTemp = 3.1415;
  targetObjectTemp.write();
  writeCh1Group1.write();
  BOOST_CHECK_CLOSE(dummy->getFloat(TecFrame::TecParameter::TemTargetObjectTemp), 0.0, 0.0001);
  testFacility.stepApplication();
  BOOST_CHECK_CLOSE(dummy->getFloat(TecFrame::TecParameter::TemTargetObjectTemp), 3.1415, 0.0001);

  targetObjectTemp = 3.5e27;
  targetObjectTemp.write();
  writeCh1Group1.write();
  BOOST_CHECK_CLOSE(dummy->getFloat(TecFrame::TecParameter::TemTargetObjectTemp), 3.1415, 0.0001);
  testFacility.stepApplication();
  BOOST_CHECK_CLOSE(dummy->getFloat(TecFrame::TecParameter::TemTargetObjectTemp), 3.5e27, 0.0001);

  targetObjectTemp = -120;
  targetObjectTemp.write();
  writeCh1Group1.write();
  BOOST_CHECK_CLOSE(dummy->getFloat(TecFrame::TecParameter::TemTargetObjectTemp), 3.5e27, 0.0001);
  testFacility.stepApplication();
  BOOST_CHECK_CLOSE(dummy->getFloat(TecFrame::TecParameter::TemTargetObjectTemp), -120, 0.0001);
}

/**********************************************************************************************************************/

BOOST_AUTO_TEST_SUITE_END()
